@@ -6,7 +6,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2020
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2022
 
 
 """
@@ -14,9 +14,11 @@ operations related to Requests.
 """
 
 import copy
+import datetime
 
 from idds.common.constants import (RequestStatus, RequestLocking, WorkStatus,
-                                   CollectionType, CollectionStatus, CollectionRelationType)
+                                   CollectionType, CollectionStatus, CollectionRelationType,
+                                   MessageStatus)
 from idds.orm.base.session import read_session, transactional_session
 from idds.orm import requests as orm_requests
 from idds.orm import transforms as orm_transforms
@@ -30,6 +32,8 @@ def create_request(scope=None, name=None, requester=None, request_type=None,
                    username=None, userdn=None, transform_tag=None,
                    status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
                    lifetime=None, workload_id=None, request_metadata=None,
+                   new_poll_period=1, update_poll_period=10,
+                   new_retries=0, update_retries=0, max_new_retries=3, max_update_retries=0,
                    processing_metadata=None):
     """
     Add a request.
@@ -57,6 +61,9 @@ def create_request(scope=None, name=None, requester=None, request_type=None,
               'username': username, 'userdn': userdn,
               'transform_tag': transform_tag, 'status': status, 'locking': locking,
               'priority': priority, 'lifetime': lifetime, 'workload_id': workload_id,
+              'new_poll_period': new_poll_period, 'update_poll_period': update_poll_period,
+              'new_retries': new_retries, 'update_retries': update_retries,
+              'max_new_retries': max_new_retries, 'max_update_retries': max_update_retries,
               'request_metadata': request_metadata, 'processing_metadata': processing_metadata}
     return orm_requests.create_request(**kwargs)
 
@@ -66,6 +73,8 @@ def add_request(scope=None, name=None, requester=None, request_type=None,
                 username=None, userdn=None, transform_tag=None,
                 status=RequestStatus.New, locking=RequestLocking.Idle, priority=0,
                 lifetime=None, workload_id=None, request_metadata=None,
+                new_poll_period=1, update_poll_period=10,
+                new_retries=0, update_retries=0, max_new_retries=3, max_update_retries=0,
                 processing_metadata=None, session=None):
     """
     Add a request.
@@ -85,7 +94,7 @@ def add_request(scope=None, name=None, requester=None, request_type=None,
 
     :returns: request id.
     """
-    if workload_id is None and request_metadata and 'workload_id' in request_metadata:
+    if workload_id is None and request_metadata and 'workload_id' in request_metadata and request_metadata['workload_id']:
         workload_id = int(request_metadata['workload_id'])
     # request_metadata = convert_request_metadata_to_workflow(scope, name, workload_id, request_type, request_metadata)
 
@@ -93,6 +102,9 @@ def add_request(scope=None, name=None, requester=None, request_type=None,
               'username': username, 'userdn': userdn,
               'transform_tag': transform_tag, 'status': status, 'locking': locking,
               'priority': priority, 'lifetime': lifetime, 'workload_id': workload_id,
+              'new_poll_period': new_poll_period, 'update_poll_period': update_poll_period,
+              'new_retries': new_retries, 'update_retries': update_retries,
+              'max_new_retries': max_new_retries, 'max_update_retries': max_update_retries,
               'request_metadata': request_metadata, 'processing_metadata': processing_metadata,
               'session': session}
     return orm_requests.add_request(**kwargs)
@@ -113,6 +125,17 @@ def get_request_ids_by_workload_id(workload_id, session=None):
     return orm_requests.get_request_ids_by_workload_id(workload_id, session=session)
 
 
+@transactional_session
+def get_request_by_id_status(request_id, status=None, locking=False, session=None):
+    req = orm_requests.get_request_by_id_status(request_id=request_id, status=status, locking=locking, session=session)
+    if req is not None and locking:
+        parameters = {}
+        parameters['locking'] = RequestLocking.Locking
+        parameters['updated_at'] = datetime.datetime.utcnow()
+        orm_requests.update_request(request_id=req['request_id'], parameters=parameters, session=session)
+    return req
+
+
 @read_session
 def get_requests(request_id=None, workload_id=None, with_detail=False,
                  with_request=False, with_transform=False, with_processing=False,
@@ -232,6 +255,7 @@ def update_request_with_transforms(request_id, parameters,
     :param new_transforms: list of transforms
     :param update_transforms: list of transforms
     """
+    new_tf_ids, update_tf_ids = [], []
     if new_transforms:
         for tf in new_transforms:
             # tf_id = orm_transforms.add_transform(**tf, session=session)
@@ -266,16 +290,17 @@ def update_request_with_transforms(request_id, parameters,
             orm_transforms.update_transform(transform_id=tf_id,
                                             parameters={'transform_metadata': tf['transform_metadata']},
                                             session=session)
-
+            new_tf_ids.append(tf_id)
     if update_transforms:
         for tr_id in update_transforms:
             orm_transforms.update_transform(transform_id=tr_id, parameters=update_transforms[tr_id], session=session)
+            update_tf_ids.append(tf_id)
 
     if new_messages:
         orm_messages.add_messages(new_messages, session=session)
     if update_messages:
         orm_messages.update_messages(update_messages, session=session)
-    return orm_requests.update_request(request_id, parameters, session=session)
+    return orm_requests.update_request(request_id, parameters, session=session), new_tf_ids, update_tf_ids
 
 
 @transactional_session
@@ -296,41 +321,22 @@ def update_request_with_workprogresses(request_id, parameters, new_workprogresse
 
 
 @transactional_session
-def get_requests_with_messaging(locking=False, bulk_size=None, session=None):
+def get_operation_request_msgs(locking=False, bulk_size=None, session=None):
     msgs = core_messages.retrieve_request_messages(request_id=None, bulk_size=bulk_size, session=session)
     if msgs:
-        req_ids = [msg['request_id'] for msg in msgs]
-        if locking:
-            req2s = orm_requests.get_requests_by_status_type(status=None, request_ids=req_ids,
-                                                             locking=locking, locking_for_update=True,
-                                                             bulk_size=None, session=session)
-            if req2s:
-                reqs = []
-                for req_id in req_ids:
-                    if len(reqs) >= bulk_size:
-                        break
-                    for req in req2s:
-                        if req['request_id'] == req_id:
-                            reqs.append(req)
-                            break
-            else:
-                reqs = []
-
-            parameters = {'locking': RequestLocking.Locking}
-            for req in reqs:
-                orm_requests.update_request(request_id=req['request_id'], parameters=parameters, session=session)
-            return reqs
-        else:
-            reqs = orm_requests.get_requests_by_status_type(status=None, request_ids=req_ids, locking=locking,
-                                                            locking_for_update=locking,
-                                                            bulk_size=bulk_size, session=session)
-            return reqs
-    else:
-        return []
+        # req_ids = [msg['request_id'] for msg in msgs]
+        to_updates = []
+        for msg in msgs:
+            to_updates.append({'msg_id': msg['msg_id'],
+                               'status': MessageStatus.Delivered})
+        core_messages.update_messages(to_updates)
+    return msgs
 
 
 @transactional_session
-def get_requests_by_status_type(status, request_type=None, time_period=None, locking=False, bulk_size=None, to_json=False, by_substatus=False, with_messaging=False, session=None):
+def get_requests_by_status_type(status, request_type=None, time_period=None, locking=False, bulk_size=None, to_json=False,
+                                by_substatus=False, not_lock=False, next_poll_at=None, new_poll=False, update_poll=False,
+                                only_return_id=False, session=None):
     """
     Get requests by status and type
 
@@ -343,21 +349,19 @@ def get_requests_by_status_type(status, request_type=None, time_period=None, loc
 
     :returns: list of Request.
     """
-    if with_messaging:
-        reqs = get_requests_with_messaging(locking=locking, bulk_size=bulk_size, session=session)
-        if reqs:
-            return reqs
-
     if locking:
-        if bulk_size:
+        if not only_return_id and bulk_size:
             # order by cannot work together with locking. So first select 2 * bulk_size without locking with order by.
             # then select with locking.
             req_ids = orm_requests.get_requests_by_status_type(status, request_type, time_period, locking=locking, bulk_size=bulk_size * 2,
                                                                locking_for_update=False, to_json=False, by_substatus=by_substatus,
+                                                               new_poll=new_poll, update_poll=update_poll,
                                                                only_return_id=True, session=session)
             if req_ids:
                 req2s = orm_requests.get_requests_by_status_type(status, request_type, time_period, request_ids=req_ids,
-                                                                 locking=locking, locking_for_update=True, bulk_size=None, to_json=to_json,
+                                                                 locking=locking, locking_for_update=True, bulk_size=None,
+                                                                 to_json=to_json,
+                                                                 new_poll=new_poll, update_poll=update_poll,
                                                                  by_substatus=by_substatus, session=session)
                 if req2s:
                     # reqs = req2s[:bulk_size]
@@ -378,13 +382,24 @@ def get_requests_by_status_type(status, request_type=None, time_period=None, loc
         else:
             reqs = orm_requests.get_requests_by_status_type(status, request_type, time_period, locking=locking, locking_for_update=locking,
                                                             bulk_size=bulk_size,
+                                                            new_poll=new_poll, update_poll=update_poll, only_return_id=only_return_id,
                                                             to_json=to_json, by_substatus=by_substatus, session=session)
 
-        parameters = {'locking': RequestLocking.Locking}
-        for req in reqs:
-            orm_requests.update_request(request_id=req['request_id'], parameters=parameters, session=session)
+        parameters = {}
+        if not not_lock:
+            parameters['locking'] = RequestLocking.Locking
+        if next_poll_at:
+            parameters['next_poll_at'] = next_poll_at
+        parameters['updated_at'] = datetime.datetime.utcnow()
+        if parameters:
+            for req in reqs:
+                if type(req) in [dict]:
+                    orm_requests.update_request(request_id=req['request_id'], parameters=parameters, session=session)
+                else:
+                    orm_requests.update_request(request_id=req, parameters=parameters, session=session)
     else:
         reqs = orm_requests.get_requests_by_status_type(status, request_type, time_period, locking=locking, bulk_size=bulk_size,
+                                                        new_poll=new_poll, update_poll=update_poll, only_return_id=only_return_id,
                                                         to_json=to_json, by_substatus=by_substatus, session=session)
     return reqs
 
