@@ -110,19 +110,33 @@ def show_works(req):
 # reqs = get_requests(request_id=299111, with_request=True, with_detail=False, with_metadata=True)
 # reqs = get_requests(request_id=299235, with_request=True, with_detail=False, with_metadata=True)
 # reqs = get_requests(request_id=965, with_request=True, with_detail=False, with_metadata=True)
-reqs = get_requests(request_id=1687, with_request=True, with_detail=False, with_metadata=True)
+# reqs = get_requests(request_id=350695, with_request=True, with_detail=False, with_metadata=True)
+
+"""
+reqs = get_requests(request_id=350723, with_request=True, with_detail=False, with_metadata=True)
 for req in reqs:
     # print(req['request_id'])
+    print(req)
     # print(rets)
-    print(json_dumps(req, sort_keys=True, indent=4))
+    # print(json_dumps(req, sort_keys=True, indent=4))
     # show_works(req)
     pass
     workflow = req['request_metadata']['workflow']
+    print(workflow.runs.keys())
+    # print(workflow.runs["1"])
+    # print(json_dumps(workflow.runs["1"], sort_keys=True, indent=4))
+
+    print(workflow.runs["1"].works.keys())
+    # print(workflow.runs["1"].works["7bdcf871"])
+    # print(json_dumps(workflow.runs["1"].works["7bdcf871"], indent=4))
+    print(workflow.runs["1"].works["7bdcf871"].runs.keys())
+    print(json_dumps(workflow.runs["1"].works["7bdcf871"].runs["2"], indent=4))
     if hasattr(workflow, 'get_relation_map'):
         # print(json_dumps(workflow.get_relation_map(), sort_keys=True, indent=4))
         pass
 
 sys.exit(0)
+"""
 
 """
 # reqs = get_requests()
@@ -138,17 +152,22 @@ sys.exit(0)
 
 """
 
-"""
-# tfs = get_transforms(request_id=241)
-tfs = get_transforms(transform_id=176320)
+
+tfs = get_transforms(request_id=350723)
+# tfs = get_transforms(transform_id=350723)
 for tf in tfs:
     # print(tf)
     # print(tf['transform_metadata']['work'].to_dict())
-    print(json_dumps(tf, sort_keys=True, indent=4))
+    # print(tf)
+    # print(json_dumps(tf, sort_keys=True, indent=4))
+    print(tf['request_id'], tf['workload_id'])
+    print(tf['transform_metadata']['work_name'])
+    print(tf['transform_metadata']['work'].num_run)
+    print(tf['transform_metadata']['work'].task_name)
     pass
 
 sys.exit(0)
-"""
+
 
 """
 msgs = retrieve_messages(workload_id=25972557)
@@ -169,8 +188,8 @@ print(number_contents)
 sys.exit(0)
 """
 
-# prs = get_processings(request_id=219)
-prs = get_processings(transform_id=176320)
+prs = get_processings(request_id=350723)
+# prs = get_processings(transform_id=350723)
 i = 0
 for pr in prs:
     # if pr['request_id'] == 91:
